-- node class (although this could be more generic, currently lots of map specific stuff)
Node = class("Node")

Node.static.radius = 35
Node.static.connectDist = 200

-- this function could have arguments, but it's not necessary in this specific case
-- the arguments would be what you pass into :new()
function Node:initialize(x, y)
	local width = love.graphics.getWidth()
	local height = love.graphics.getHeight()

	self.x = x-- or math.random(width/8, 7*width/8)
	self.y = y-- + math.random(-50, 50) or math.random(height/8, 7*height/8)
	self.radius = Node.radius + math.random(-10, 10)
	self.connections = {}

	-- types:
	--	normal
	--	town
	self.type = "normal"

	self.color = {235, 33, 56}
	self.townColor = {20, 133, 204}
	self.exitColor = {141, 0, 222}
	self.border = {255, 255, 255}
end

function Node:distTo(other)
	return math.sqrt((other.x - self.x)^2 + (other.y - self.y)^2)
end

-- tests whether another node is connected to this one
function Node:connectedTo(other)
	for k, node in pairs(self.connections) do
		-- it is connected
		if node == other then
			return true
		end
	end

	-- it is not connected
	return false
end

-- connects two nodes
function Node:connect(other)
	table.insert(self.connections, other)
	table.insert(other.connections, self)
end

function Node:draw()
	if self.type == "town" then
		love.graphics.setColor(self.townColor)
	elseif self.type == "exit" then
		love.graphics.setColor(self.exitColor)
	else
		love.graphics.setColor(self.color)
	end
	love.graphics.circle("fill", self.x, self.y, self.radius)

	love.graphics.setColor(self.border)
	love.graphics.circle("line", self.x, self.y, self.radius)
end


NodeGroup = class("NodeGroup")

function NodeGroup:initialize()
	self.activeNode = 0
	self.list = {}
end

function NodeGroup:generateMap()
-- basic line node gen
--[[
	for i = 1, 10 do
		local x = i * 101
		local y = love.graphics.getHeight()/2-Node.radius/2 + math.random(-50, 50)
		table.insert(self.list, Node:new(x, y))
	end
]]

-- node gen based on a bezier curve
--[[
	local width = love.graphics.getWidth()
	local height = love.graphics.getHeight()
	local points = 10
	local margin = width / 40
	local sep = (love.graphics.getWidth() - margin) / points
	local Points = {}
	for i = 1, points do
		table.insert(Points, margin+sep*i)
		table.insert(Points, height/2+math.random(height*3/5)-height*3/10)
	end
	
	local bezier = love.math.newBezierCurve(Points)
	for i = 0, 9 do
		local x, y = bezier:evaluate(i/10)
		table.insert(self.list, Node:new(x, y))
	end
]]

	local width = love.graphics.getWidth()
	local height = love.graphics.getHeight()
	local distMin = 80
	local distMax = 150
	local nodes = {}
	table.insert(nodes, {math.random(width/4, width*3/4), math.random((height-200)*3/4, (height-200)*3/4) + 100})
	
	local minNodes = 5
	
	repeat
		local nodeNum = math.random(25, 30)
		if game.mapLevel then
			nodeNum = math.random(20+game.mapLevel*2, 30+game.mapLevel*3)
		end
		
		for i = 1, nodeNum do
			local num = math.random(#nodes)
			local x, y = nodes[num][1], nodes[num][2]
			local dist = math.random(distMin, distMax)
			local x2, y2 = x + math.cos(math.random(2*math.pi)) * dist, y + math.sin(math.random(2*math.pi)) * dist
			
			local ok = true
			for j = 1, #nodes do
				local x3, y3 = nodes[j][1], nodes[j][2]
				if math.dist(x2, y2, x3, y3) <= 119 then
					ok = false
				end
			end
			
			if ok then
				table.insert(nodes, {x2, y2})
			end
		end
		
	until #nodes >= minNodes
	
	for i = 1, #nodes do
		local x, y = nodes[i][1], nodes[i][2]
		table.insert(self.list, Node:new(x, y))
	end
	
	self:connect()
	
	for i = 1, #self.list do
	
	
	end
	
	
	local exitNode = 2
	repeat
		local k = true
		exitNode = math.random(2, #self.list)
		for i = 1, #self.list[exitNode].connections do
			if self.list[exitNode].connections[i] == self.list[1] then
				k = false
			end
		end
	until k == true
	self.list[exitNode].type = "exit"
	
	local shopNum = math.ceil(#self.list/10)
	for i = 1, shopNum do
		repeat
			local k = true
			local n = math.random(2, #self.list)
			if self.list[n].type ~= "exit" then
				self.list[n].type = "town"
				k = false
			end
		until k == false
	end
end

function NodeGroup:empty()
	self.list = {}
end

function NodeGroup:toSerialize()
    local data = {}

    for k, node in pairs(self.list) do
        table.insert(data, {
        	k = k,
            x = node.x,
            y = node.y,
            radius = node.radius,
            type = node.type,
        })
    end

    return data
end

function NodeGroup:fromSerialize(data)
	for k, node in pairs(data) do
		self.list[node.k] = Node:new(node.x, node.y)
		self.list[node.k].radius = node.radius
		self.list[node.k].type = node.type
	end

	self:connect()
end

function NodeGroup:connect()
	for k, node in pairs(self.list) do
		for j, node2 in pairs(self.list) do
			if node ~= node2 then
				if node:distTo(node2) <= Node.connectDist then
					node:connect(node2)
				end
			end
		end
	end
end

function NodeGroup:closestNode(x, y, threshold)
	x = x or love.mouse.getX()
	y = y or love.mouse.getY()
	threshold = threshold or node.radius

	local min = math.huge
	local minNode = nil

	for k, node in pairs(self.list) do
		local d = math.dist(x, y, node.x, node.y)
		if d <= threshold and d < min then
			min = d
			minNode = node
		end
	end

	return minNode, min
end

function NodeGroup:update()
	--[[local mouseX, mouseY = love.mouse.getPosition()
	
	if nodes.activeNode ~= 0 then
		local activeNode = nodes.list[nodes.activeNode]
		if node:distTo({x = mouseX, y = mouseY}) > node.radius then
			activeNode.color = {235, 33, 56}
			nodes.activeNode = 0
		end
	end
	
	if nodes.activeNode == 0 then
		for k, node in pairs(nodes.list) do
			if node:distTo({x = mouseX, y = mouseY}) <= node.radius then
				nodes.activeNode = k
				nodes.list[nodes.activeNode].color = {247, 67, 87}
				break
			end
		end
	end]]
end

function NodeGroup:keypressed(key, isrepeat)
	local currentNode = game.player.location
	local x, y = currentNode.x, currentNode.y
	local nodesConnected = currentNode.connections
	--local dir = key == "up" and 270 or key == "left" and 180 or key == "down" and 90 or key == "right" and 0
	local dir = 270
	if key == "left" then dir = 180 elseif key == "down" then dir = 90 elseif key == "right" then dir = 0 end
	
	local closestToDif = 100000
	local closestToNode = 0
	for i = 1, #nodesConnected do
		local x2, y2 = nodesConnected[i].x, nodesConnected[i].y
		local angle = math.deg(math.angle(x, y, x2, y2))
		
		local angleDif = math.abs(dir - angle) 
		if angleDif < closestToDif then
			closestToDif = angleDif
			closestToNode = i
		end
	end
	
	if game.player.lastTween == nil then
		game.player:moveTo(nodesConnected[closestToNode])
	end
end

function NodeGroup:draw()
	love.graphics.setLineWidth(10)
	for k, node in pairs(self.list) do

		-- draw connections for each node
		for i, conn in pairs(node.connections) do
			love.graphics.setColor(242, 242, 109)
			love.graphics.line(node.x, node.y, conn.x, conn.y)
		end
	end

	love.graphics.setLineWidth(4)
	for k, node in pairs(self.list) do
		node:draw()
	end
end