Selector = class("Selector")

-- List should contain objects with x, y, width, and height values. The selector can iterate through all of these objects
-- Color is a table {r, g, b, [a]}
-- Current is the currently selected item in List (defaults to 1)
-- Segmented is a boolean (whether the selector should use quads or not). Use false for a single image selector
-- Blink is a boolean, if true then the selector will blink on and off at a fixed rate
function Selector:initialize(list, current, orientation, segmented, blink, color, img)
	self.list = list
    self.current = current or 1
    if self.list[self.current] ~= nil then
    	self.currentElement = self.list[self.current]
    end
    self.orientation = orientation or "horizontal"
	self.segmented = segmented or true
	self.blink = blink or false
	self.color = color or {255, 255, 255, 255}

	self.unselectable = {}
	
	if img == nil then
		self.img = love.graphics.newImage('img/ui/selector.png')
	elseif type(img) == "string" then
		self.img = love.graphics.newImage(img)
	elseif img:type() == "Image" then
		self.img = img
	end
	self.imgWidth, self.imgHeight = self.img:getDimensions()
	
	if self.segmented then
		self.horizSegments = 3
		self.vertSegments = 2
		
		self.segmentWidth = self.imgWidth / self.horizSegments
		self.segmentHeight = self.imgHeight / self.vertSegments
		
		
		self.handleTopLeft = love.graphics.newQuad(0, 0, self.segmentWidth, self.segmentHeight, self.imgWidth, self.imgHeight)
		self.handleTop = love.graphics.newQuad(self.segmentWidth, 0, self.segmentWidth, self.segmentHeight, self.imgWidth, self.imgHeight)
		self.handleTopRight = love.graphics.newQuad(self.segmentWidth*2, 0, self.segmentWidth, self.segmentHeight, self.imgWidth, self.imgHeight)
		self.handleBottomLeft = love.graphics.newQuad(0, self.segmentHeight, self.segmentWidth, self.segmentHeight, self.imgWidth, self.imgHeight)
		self.handleBottom = love.graphics.newQuad(self.segmentWidth, self.segmentHeight, self.segmentWidth, self.segmentHeight, self.imgWidth, self.imgHeight)
		self.handleBottomRight = love.graphics.newQuad(self.segmentWidth*2, self.segmentHeight, self.segmentWidth, self.segmentHeight, self.imgWidth, self.imgHeight)
	else
		self.horizSegments = 1
		self.vertSegments = 1
		
		self.segmentWidth = self.imgWidth / self.horizSegments
		self.segmentHeight = self.imgHeight / self.vertSegments
	end
	
	self:setPos()
	
	if self.blink then
		self.timer = 0
		self.timerMax = .7
		
		self.state = "on"
	end
end

function Selector:setPos()
	if self.segmented then
		self.segments = math.ceil(self.currentElement.width / self.segmentWidth) + 1
	else
		self.segments = 1
	end
	
	self.width = self.segments*self.segmentWidth
    self.x = self.currentElement.x + self.currentElement.width/2 - self.width/2
    self.y = self.currentElement.y
	self.height = self.currentElement.height
end

function Selector:keypressed(input, isrepeat)
	if input == "right" or input == "left" or input == "up" or input == "down" then
		self:move(input)
	elseif input == " " or input == "return" or input == "z" then
	    if self.currentElement:isInstanceOf(Button) then
            self.currentElement:activated()
        end
    elseif input == "tab" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) then
    	self:move("left")
    elseif input == "tab" then
    	self:move("right")
    end
end

function Selector:selectNext()
	self.current = self.current < #self.list and self.current+1 or 1
	self.currentElement = self.list[self.current]

	for k, cantSelect in pairs(self.unselectable) do
		if self.current == cantSelect then
			self:selectNext()
		end
	end
end

function Selector:selectPrev()
	self.current = self.current > 1 and self.current-1 or #self.list
	self.currentElement = self.list[self.current]

	for k, cantSelect in pairs(self.unselectable) do
		if self.current == cantSelect then
			self:selectPrev()
		end
	end
end

function Selector:move(input)
    -- reset blink, so the selector is always visible right when you move
    self.state = "on"
    self.timer = 0

    -- deselect current button
    self.currentElement.selected = false

    if self.orientation == "horizontal" then
		if input == "right" or input == "up" then
			self:selectNext()
		elseif input == "left" or input == "down" then
			self:selectPrev()
		end
	elseif self.orientation == "vertical" then
		if input == "right" or input == "down" then
			self:selectNext()
		elseif input == "left" or input == "up" then
		    self:selectPrev()
		end
	end
	
	self:setPos()

    -- select current button
    self.currentElement.selected = true

	return self.current
end

function Selector:update(dt)
	if self.blink then
		self.timer = self.timer + dt
		if self.timer > self.timerMax then
			self.state = self.state == "on" and "off" or "on"
			self.timer = self.timer - self.timerMax
		end
	end

	-- when you press enter on an input, skip to the next element
	if self.currentElement:isInstanceOf(Input) then
		if not self.currentElement.selected then
			self:move("right")
		end
	end
end

function Selector:draw()
	local oldColor = {love.graphics.getColor()}

	if self.blink and self.state == "on" or not self.blink then
		local x, y = self.x, self.y
		
		love.graphics.setColor(self.color)
		
		if self.segmented then
			love.graphics.draw(self.img, self.handleTopLeft, x, y-3)
			love.graphics.draw(self.img, self.handleBottomLeft, x, y+self.height-self.segmentHeight+3)
			
			for i = 1, self.segments - 2 do
				love.graphics.draw(self.img, self.handleTop, x+i*self.segmentWidth, y-3)
				love.graphics.draw(self.img, self.handleBottom, x+i*self.segmentWidth, y+self.height-self.segmentHeight+3)
			end
			
			love.graphics.draw(self.img, self.handleTopRight, x+((self.segments-1)*self.segmentWidth), y-3)
			love.graphics.draw(self.img, self.handleBottomRight, x+((self.segments-1)*self.segmentWidth), y+self.height-self.segmentHeight+3)
		else

			love.graphics.setColor(255, 255, 255)
			love.graphics.draw(self.img, self.x, self.y)

		end
	end
	
	love.graphics.setColor(oldColor)
end