-- Function: searchTable
-- Returns the index of a value in a table. If the value
-- does not exist in the table, this returns nil.
--
-- Arguments:
--      table - table to search
--      search - value to search for
--
-- Returns:
--      integer index or nil

function searchTable (table, search)
    for i, value in ipairs(table) do
        if value == search then return i end
    end

    return nil
end

-- Function: shuffleTable
-- Shuffles the contents of a table in place.
-- via http://www.gammon.com.au/forum/?id=9908
--
-- Arguments:
--      src - table to shuffle
--
-- Returns:
--      table passed

function shuffleTable (src)
  local n = #src
 
  while n >= 2 do
    -- n is now the last pertinent index
    local k = math.random(n) -- 1 <= k <= n
    -- Quick swap
    src[n], src[k] = src[k], src[n]
    n = n - 1
  end
 
  return src
end

-- Function: dump
-- Returns a string representation of a variable in a way
-- that can be reconstituted via loadstring(). Yes, this
-- is basically a serialization function, but that's so much
-- to type :) This ignores any userdata, functions, or circular
-- references.
-- via http://www.lua.org/pil/12.1.2.html
--
-- Arguments:
--      source - variable to describe
--      ignore - a table of references to ignore (to avoid cycles),
--               defaults to empty table. This uses the references as keys,
--               *not* as values, to speed up lookup.
--
-- Returns:
--      string description

function dump (source, ignore)
    ignore = ignore or { source = true }
    local sourceType = type(source)
    
    if sourceType == 'table' then
        local result = '{ '

        for key, value in pairs(source) do
            if not ignore[value] then
                if type(value) == 'table' then
                    ignore[value] = true
                end

                local dumpValue = dump(value, ignore)

                if dumpValue ~= '' then
                    -- don't turn numerical keys into strings
                    if type(key) == 'number' then
                        result = result .. '[' .. key .. '] = ' .. dumpValue .. ', '
                    else
                        result = result .. '["' .. key .. '"] = ' .. dumpValue .. ', '
                    end
                end
            end
        end

        if result ~= '{ ' then
            return string.sub(result, 1, -3) .. ' }'
        else
            return '{}'
        end
    elseif sourceType == 'string' then
        return string.format('%q', source)
    elseif sourceType ~= 'userdata' and sourceType ~= 'function' then
        return tostring(source)
    else
        return ''
    end
end

-- Function: sound
-- Loads a sound but does not play it. It's important to use the hint property
-- appropriately; if set incorrectly, it can cause sound playback to stutter or lag.
--
-- Arguments:
--      path - string pathname to sound
--      hint - either 'static' or 'stream', depending on length of sound
--
-- Returns:
--      LOVE sound source, see https://love2d.org/wiki/Source

function sound (path, hint)
    return love.audio.newSource(path, hint)
end

-- Function: playSound
-- Plays a sound once. This is the easiest way to play a sound. It's important
-- to use the hint property appropriately; if set incorrectly, it can cause sound
-- playback to stutter or lag.
--
-- Arguments:
--      path - string pathname to sound
--      volume - volume to play at, from 0 to 1; default 1
--      hint - either 'short' or 'long', depending on length of sound; default 'short'
--
-- Returns:
--      LOVE sound source, see https://love2d.org/wiki/Source

function playSound (path, volume, hint)
    volume = volume or 1

    if type(path) == "string" then
        local source = sound(path, hint)
        source:setVolume(volume)
        source:play()
        return source
    else
        path:setVolume(volume)
        path:play()
        return path
    end
end

function serializeTable(val, name, skipnewlines, depth)
    skipnewlines = skipnewlines or false
    depth = depth or 0

    local tmp = string.rep(" ", depth)

    if name then tmp = tmp .. name .. " = " end

    if type(val) == "table" then
        tmp = tmp .. "{" .. (not skipnewlines and "\n" or "")

        for k, v in pairs(val) do
            tmp =  tmp .. serializeTable(v, k, skipnewlines, depth + 1) .. "," .. (not skipnewlines and "\n" or "")
        end

        tmp = tmp .. string.rep(" ", depth) .. "}"
    elseif type(val) == "number" then
        tmp = tmp .. tostring(val)
    elseif type(val) == "string" then
        tmp = tmp .. string.format("%q", val)
    elseif type(val) == "boolean" then
        tmp = tmp .. (val and "true" or "false")
    else
        tmp = tmp .. "\"[inserializeable datatype:" .. type(val) .. "]\""
    end

    return tmp
end

table.reduce = function (list, fn) 
    local acc
    for k, v in ipairs(list) do
        if 1 == k then
            acc = v
        else
            acc = fn(acc, v)
        end 
    end 
    return acc 
end